module vga
(
    input               CLOCK,
    output  reg  [3:0]  R,
    output  reg  [3:0]  G,
    output  reg  [3:0]  B,
    output              HS,
    output              VS,
    output  reg  [11:0] VGA_FA,
    output  reg  [10:0] VGA_CA,
    input        [ 7:0] VGA_FQ,
    input        [ 7:0] VGA_CQ
);

// ---------------------------------------------------------------------
// Тайминги для горизонтальной и вертикальной развертки
//           Visible       Front        Sync        Back       Whole
parameter hzv = 1440, hzf =   80, hzs =  152, hzb =  232, hzw = 1904,
          vtv =  900, vtf =    1, vts =    3, vtb =   28, vtw =  932;
// ---------------------------------------------------------------------
assign HS = X  < (hzb + hzv + hzf); // NEG.
assign VS = Y >= (vtb + vtv + vtf); // POS.
// ---------------------------------------------------------------------
// Позиция луча в кадре и максимальные позиции (x,y)
reg  [10:0] X = 0;
reg  [ 9:0] Y = 0;
wire        xmax = (X == hzw - 1);
wire        ymax = (Y == vtw - 1);
wire [10:0] x    = (X - hzb); // x=[0..1439]
wire [ 9:0] y    = (Y - vtb); // y=[0..899]
wire [10:0] xm   = x - 80 + 16;
wire [ 9:0] ym   = y - 50;
// ---------------------------------------------------------------------
reg  [ 7:0] char, mask;
wire        paper   = (x >= 80 && x < 80+1280 && y >= 50 && y < 850);
wire        pix     = mask[ ~xm[3:1] ];
// ---------------------------------------------------------------------

always @(posedge CLOCK) begin

    // Черный цвет за пределами видимой области рисования
    {R, G, B} <= 12'h000;

    // Кадровая развертка
    X <= xmax ?         0 : X + 1;
    Y <= xmax ? (ymax ? 0 : Y + 1) : Y;

    // Генерация изображения
    case (x[3:0])
    0:  begin VGA_CA <= 80*ym[9:5] + xm[10:4]; end
    // 1: Защелкнуть A
    13: begin VGA_FA <= {VGA_CQ, ym[4:1]}; end
    // 14: Защелкивание A
    15: begin mask   <= VGA_FQ; end
    endcase

    // Вывод окна видеоадаптера
    if (X >= hzb && X < hzb + hzv && Y >= vtb && Y < vtb + vtv)
    begin
        {R, G, B} <= paper ? (pix ? 12'h4A4 : 12'h000) : (ym[1:0] ? 12'h000 : 12'h111);
    end

end

endmodule
